//-----------------------------------------------------------------------------
// Torque Game Engine 
// Copyright (C) GarageGames.com, Inc.
//-----------------------------------------------------------------------------

// This file contains Weapon and Ammo Class/"namespace" helper methods
// as well as hooks into the inventory system. These functions are not
// attached to a specific C++ class or datablock, but define a set of
// methods which are part of dynamic namespaces "class". The Items
// include these namespaces into their scope using the  ItemData and
// ItemImageData "className" variable.

// All ShapeBase images are mounted into one of 8 slots on a shape.
// This weapon system assumes all primary weapons are mounted into
// this specified slot:
$WeaponSlot = 0;

//-----------------------------------------------------------------------------
// Weapon Class 
//-----------------------------------------------------------------------------

//generic weapon sounds
datablock AudioProfile(weaponSwitchSound)
{
   filename    = "~/data/sound/weapons/switch.wav";
   description = AudioClosest3d;
   preload = true;
};


function Weapon::onUse(%data, %player, %InvPosition)
{
	//if the image is mounted already, unmount it
	//if it isnt, mount it

	%mountPoint = %data.image.mountPoint;
	%mountedImage = %player.getMountedImage(%mountPoint); 

	//new system, mount the image no matter what
	%image = %data.image;
	
	%player.mountimage(%image, %mountPoint);
}

function Weapon::onPickup(%data, %obj, %player, %amount)
{
   //static weapons come with full ammo
   %ammo = %data.ammo;
   if(%obj.isStatic() && isObject(%ammo))
   {
      //give player full ammo for this weapon
      %playerData = %player.getDataBlock();
      %player.inv[%ammo.getName()] = %playerData.maxInv[%ammo.getName()];

      //set weapon as having ammo
      %ammo.onInventory(%player,%player.inv[%ammo.getName()]);

      //tell client that they got some ammo
      %client = %player.client;
      if(isObject(%client))
         messageClient(%client, 'InventoryAmount', '', %ammo.getID(), %player.inv[%ammo.getName()]);
   }

   //pickup the actual weapon
	Parent::OnPickup(%data,%obj,%player, %amount);
}

function Weapon::onInventory(%data,%obj,%amount)
{
   // Weapon inventory has changed, make sure there are no weapons
   // of this type mounted if there are none left in inventory.
   if (!%amount && (%slot = %obj.getMountSlot(%data.image)) != -1)
      %obj.unmountImage(%slot);
}   


//-----------------------------------------------------------------------------
// Weapon Image Class
//-----------------------------------------------------------------------------
function WeaponImage::onFire(%data, %obj, %slot)
{
   if(%obj.invincible)
      %obj.schedule(150, setInvincible, false);

   if(isObject(%data.ammo))
      %obj.decInventory( %data.ammo, 1 );

   // ----

   // -- required for projectiles which inherit velocity from their parent
   %muzzleVelocity = VectorAdd(
      VectorScale(%obj.getMuzzleVector(%slot), %data.projectile.muzzleVelocity),
      VectorScale(%obj.getVelocity(), %data.projectile.velInheritFactor));

   // -- if there's a projectile variance, apply it
   if( %data.projectileSpread )
   {
      for(%i = 0; %i < 3; %i++)
         %matrix = %matrix @ (getRandom() - 0.5) * 2 * 3.1415926 * %data.projectileSpread @ " ";
      %matrix = MatrixCreateFromEuler( %matrix );
      %muzzleVelocity = MatrixMulVector( %matrix, %muzzleVelocity );
   }
   //echo("data = ", %data);
   //echo("projectiledata = ", %data.projectile);
   %p = new (%data.projectileType)()
   {
      dataBlock = %data.projectile;
      initialVelocity = %muzzleVelocity;
      initialPosition = %obj.getMuzzlePoint(%slot);
      sourceObject = %obj;
      sourceSlot = %slot;
      client = %obj.client;
   };
   MissionCleanup.add(%p);

   %obj.client.incStat("RoundsFired_" @ %data.item.hudName);

   if(isObject(%data.ammo))
      if(%obj.getInventory(%data.ammo) <= 0) 
         %obj.setImageAmmo(%slot, 0);

   return(%p);
}

function WeaponImage::onMount(%data, %obj, %slot)
{
   // Images assume a false ammo state on load.  We need to
   // set the state according to the current inventory.

   if(getSimTime() - %obj.spawnTime < $Game::SpawnFireDelay)
      %obj.setImageLoaded(%slot, false);
   else
      %obj.setImageLoaded(%slot, true);
   
	if(isObject(%data.ammo))	//dont check ammo if the image doesn't use ammo
	{
		if (%obj.getInventory(%data.ammo) > 0)
		   %obj.setImageAmmo(%slot,true);
      else
         %obj.setImageAmmo(%slot,false);
	}
	else
	{
		%obj.setImageAmmo(%slot,true);
	}
}

function WeaponImage::onUnmount(%data,%obj,%slot)
{

}


//-----------------------------------------------------------------------------
// Ammmo Class
//-----------------------------------------------------------------------------

function Ammo::onInventory(%data,%obj,%amount)
{
   // The ammo inventory state has changed, we need to update any
   // mounted images using this ammo to reflect the new state.
   for (%i = 0; %i < 8; %i++)
   {
      if ((%image = %obj.getMountedImage(%i)) == 0)
         continue;


      if (isObject(%image.ammo) && %image.ammo.getId() == %data.getId())
         %obj.setImageAmmo(%i,%amount != 0);
   }
}   


function weaponImage::OnPlayerDeath()
{

}

function weaponImage::OnPlayerRemove()
{

}